// This may look like C code, but it's really -*- C++ -*-
/*
 * Copyright (C) 2014 Emweb bv, Herent, Belgium.
 *
 * See the LICENSE file for terms of use.
 */
#ifndef WT_DBO_STRING_STREAM_H_
#define WT_DBO_STRING_STREAM_H_

#include <Wt/Dbo/WDboDllDefs.h>

#define WT_DBO_STRINGSTREAM
#include "../WStringStream.h"
#undef WT_DBO_STRINGSTREAM

#endif // WT_DBO_STRING_STREAM_H_
