// This may look like C code, but it's really -*- C++ -*-
/*
 * Copyright (C) 2008 Emweb bv, Herent, Belgium.
 *
 * See the LICENSE file for terms of use.
 */
#ifndef DRAGEXAMPLE_H_
#define DRAGEXAMPLE_H_

#include <Wt/WContainerWidget.h>

using namespace Wt;

/// Class demonstrating drag and drop in Wt
class DragExample: public WContainerWidget
{
 public:
  DragExample();
};

#endif

